@@ -84,7 +84,7 @@ private:
 	void loadOrGenerateCover(const GameList::Entry* ge);
 	void invalidateCoverForPath(const std::string& path);
 
-	static QString formatTimespan(time_t timespan);
+	static QString formatTimespan(const time_t timespan);
 
 	float m_cover_scale = 0.0f;
 	std::atomic<u32> m_cover_scale_counter{0};
