@@ -1,19 +1,5 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
-
-#include "PrecompiledHeader.h"
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #include <QtGui/QDrag>
 #include <QtWidgets/QFileDialog>
@@ -24,58 +10,59 @@
 
 #include "common/StringUtil.h"
 
-#include "MemoryCardSettingsWidget.h"
-#include "CreateMemoryCardDialog.h"
-#include "QtHost.h"
 #include "MemoryCardConvertDialog.h"
+#include "MemoryCardCreateDialog.h"
+#include "MemoryCardSettingsWidget.h"
+#include "QtHost.h"
 #include "QtUtils.h"
 #include "SettingWidgetBinder.h"
-#include "SettingsDialog.h"
+#include "SettingsWindow.h"
 
-#include "pcsx2/MemoryCardFile.h"
+#include "pcsx2/SIO/Memcard/MemoryCardFile.h"
+
+static constexpr const char* CONFIG_SECTION = "MemoryCards";
 
 static std::string getSlotFilenameKey(u32 slot)
 {
 	return StringUtil::StdStringFromFormat("Slot%u_Filename", slot + 1);
 }
 
-MemoryCardSettingsWidget::MemoryCardSettingsWidget(SettingsDialog* dialog, QWidget* parent)
-	: QWidget(parent)
-	, m_dialog(dialog)
+MemoryCardSettingsWidget::MemoryCardSettingsWidget(SettingsWindow* settings_dialog, QWidget* parent)
+	: SettingsWidget(settings_dialog, parent)
 {
-	SettingsInterface* sif = m_dialog->getSettingsInterface();
+	SettingsInterface* sif = dialog()->getSettingsInterface();
 
-	m_ui.setupUi(this);
+	setupTab(m_ui);
 
 	// this is a bit lame, but resizeEvent() isn't good enough to autosize our columns,
 	// since the group box hasn't been resized at that point.
 	m_ui.cardGroupBox->installEventFilter(this);
-	
-	SettingWidgetBinder::BindWidgetToFolderSetting(sif, m_ui.directory, m_ui.browse, m_ui.open, m_ui.reset, "Folders", "MemoryCards", Path::Combine(EmuFolders::DataRoot, "memcards"));
-	SettingWidgetBinder::BindWidgetToBoolSetting(sif, m_ui.autoEject, "EmuCore", "McdEnableEjection", true);
+
+	SettingWidgetBinder::BindWidgetToFolderSetting(sif, m_ui.directory, m_ui.browse, m_ui.open, m_ui.reset, "Folders",
+		"MemoryCards", Path::Combine(EmuFolders::DataRoot, "memcards"));
 	SettingWidgetBinder::BindWidgetToBoolSetting(sif, m_ui.automaticManagement, "EmuCore", "McdFolderAutoManage", true);
 
 	setupAdditionalUi();
 
 	connect(m_ui.directory, &QLineEdit::textChanged, this, &MemoryCardSettingsWidget::refresh);
 	m_ui.cardList->setContextMenuPolicy(Qt::CustomContextMenu);
-	connect(m_ui.cardList, &MemoryCardListWidget::itemSelectionChanged, this, &MemoryCardSettingsWidget::updateCardActions);
-	connect(m_ui.cardList, &MemoryCardListWidget::customContextMenuRequested, this, &MemoryCardSettingsWidget::listContextMenuRequested);
+	connect(
+		m_ui.cardList, &MemoryCardListWidget::itemSelectionChanged, this, &MemoryCardSettingsWidget::updateCardActions);
+	connect(m_ui.cardList, &MemoryCardListWidget::customContextMenuRequested, this,
+		&MemoryCardSettingsWidget::listContextMenuRequested);
 
 	connect(m_ui.refreshCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::refresh);
 	connect(m_ui.createCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::createCard);
-	connect(m_ui.duplicateCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::duplicateCard);
 	connect(m_ui.renameCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::renameCard);
 	connect(m_ui.convertCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::convertCard);
 	connect(m_ui.deleteCard, &QPushButton::clicked, this, &MemoryCardSettingsWidget::deleteCard);
 
 	refresh();
 
-	dialog->registerWidgetHelp(m_ui.autoEject, tr("Auto-eject memory cards when loading save states"), tr("Checked"),
-		tr("Avoids broken memory card saves. May not work with some games such as Guitar Hero."));
-
-	dialog->registerWidgetHelp(m_ui.automaticManagement, tr("Automatically manage saves based on running game"), tr("Checked"),
-		tr("(Folder type only / Card size: Auto) Loads only the relevant booted game saves, ignoring others. Avoids running out of space for saves."));
+	dialog()->registerWidgetHelp(m_ui.automaticManagement, tr("Automatically manage saves based on running game"),
+		tr("Checked"),
+		tr("(Folder type only / Card size: Auto) Loads only the relevant booted game saves, ignoring others. Avoids "
+		   "running out of space for saves."));
 }
 
 MemoryCardSettingsWidget::~MemoryCardSettingsWidget() = default;
@@ -99,32 +86,36 @@ void MemoryCardSettingsWidget::setupAdditionalUi()
 	for (u32 i = 0; i < static_cast<u32>(m_slots.size()); i++)
 		createSlotWidgets(&m_slots[i], i);
 
-	// button to swap memory cards
-	QToolButton* swap_button = new QToolButton(m_ui.portGroupBox);
+	// button to swap Memory Cards
+	QToolButton* swap_button = new QToolButton(m_ui.slotGroupBox);
 	swap_button->setIcon(QIcon::fromTheme("arrow-left-right-line"));
 	swap_button->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
 	swap_button->setToolTip(tr("Swap Memory Cards"));
 	connect(swap_button, &QToolButton::clicked, this, &MemoryCardSettingsWidget::swapCards);
-	static_cast<QGridLayout*>(m_ui.portGroupBox->layout())->addWidget(swap_button, 0, 1);
+	static_cast<QGridLayout*>(m_ui.slotGroupBox->layout())->addWidget(swap_button, 0, 1);
 }
 
 void MemoryCardSettingsWidget::createSlotWidgets(SlotGroup* port, u32 slot)
 {
-	port->root = new QWidget(m_ui.portGroupBox);
+	const bool perGame = dialog()->isPerGameSettings();
 
-	SettingsInterface* sif = m_dialog->getSettingsInterface();
-	port->enable = new QCheckBox(tr("Port %1").arg(slot + 1), port->root);
-	SettingWidgetBinder::BindWidgetToBoolSetting(sif, port->enable, "MemoryCards", StringUtil::StdStringFromFormat("Slot%u_Enable", slot + 1), true);
-	connect(port->enable, &QCheckBox::stateChanged, this, &MemoryCardSettingsWidget::refresh);
+	port->root = new QWidget(m_ui.slotGroupBox);
+
+	SettingsInterface* sif = dialog()->getSettingsInterface();
+	port->enable = new QCheckBox(tr("Slot %1").arg(slot + 1), port->root);
+	SettingWidgetBinder::BindWidgetToBoolSetting(
+		sif, port->enable, CONFIG_SECTION, StringUtil::StdStringFromFormat("Slot%u_Enable", slot + 1), true);
+	connect(port->enable, &QCheckBox::checkStateChanged, this, &MemoryCardSettingsWidget::refresh);
 
 	port->eject = new QToolButton(port->root);
-	port->eject->setIcon(QIcon::fromTheme("eject-line"));
+	port->eject->setIcon(QIcon::fromTheme(perGame ? QStringLiteral("delete-back-2-line") : QStringLiteral("eject-line")));
 	port->eject->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Expanding);
-	port->eject->setToolTip(tr("Eject Memory Card"));
+	port->eject->setToolTip(perGame ? tr("Reset") : tr("Eject Memory Card"));
 	connect(port->eject, &QToolButton::clicked, this, [this, slot]() { ejectSlot(slot); });
 
 	port->slot = new MemoryCardSlotWidget(port->root);
-	connect(port->slot, &MemoryCardSlotWidget::cardDropped, this, [this, slot](const QString& card) { tryInsertCard(slot, card); });
+	connect(port->slot, &MemoryCardSlotWidget::cardDropped, this,
+		[this, slot](const QString& card) { tryInsertCard(slot, card); });
 
 	QHBoxLayout* bottom_layout = new QHBoxLayout();
 	bottom_layout->setContentsMargins(0, 0, 0, 0);
@@ -136,7 +127,7 @@ void MemoryCardSettingsWidget::createSlotWidgets(SlotGroup* port, u32 slot)
 	vert_layout->addWidget(port->enable, 0);
 	vert_layout->addLayout(bottom_layout, 1);
 
-	static_cast<QGridLayout*>(m_ui.portGroupBox->layout())->addWidget(port->root, 0, (slot != 0) ? 2 : 0);
+	static_cast<QGridLayout*>(m_ui.slotGroupBox->layout())->addWidget(port->root, 0, (slot != 0) ? 2 : 0);
 }
 
 void MemoryCardSettingsWidget::autoSizeUI()
@@ -147,32 +138,35 @@ void MemoryCardSettingsWidget::autoSizeUI()
 void MemoryCardSettingsWidget::tryInsertCard(u32 slot, const QString& newCard)
 {
 	// handle where the card is dragged in from explorer or something
-	const int lastSlashPos = std::max(newCard.lastIndexOf('/'), newCard.lastIndexOf('\\'));
-	const std::string newCardStr((lastSlashPos >= 0) ? newCard.mid(0, lastSlashPos).toStdString() : newCard.toStdString());
+	const qsizetype lastSlashPos = std::max(newCard.lastIndexOf('/'), newCard.lastIndexOf('\\'));
+	const std::string newCardStr(
+		(lastSlashPos >= 0) ? newCard.mid(0, lastSlashPos).toStdString() : newCard.toStdString());
 	if (newCardStr.empty())
 		return;
 
 	// make sure it's a card in the directory
 	const std::vector<AvailableMcdInfo> mcds(FileMcd_GetAvailableCards(true));
-	if (std::none_of(mcds.begin(), mcds.end(), [&newCardStr](const AvailableMcdInfo& mcd) { return mcd.name == newCardStr; }))
+	if (std::none_of(
+			mcds.begin(), mcds.end(), [&newCardStr](const AvailableMcdInfo& mcd) { return mcd.name == newCardStr; }))
 	{
-		QMessageBox::critical(this, tr("Error"), tr("This memory card is unknown."));
+		QMessageBox::critical(this, tr("Error"), tr("This Memory Card cannot be recognized or is not a valid file type."));
 		return;
 	}
 
-	m_dialog->setStringSettingValue("MemoryCards", getSlotFilenameKey(slot).c_str(), newCardStr.c_str());
+	dialog()->setStringSettingValue(CONFIG_SECTION, getSlotFilenameKey(slot).c_str(), newCardStr.c_str());
 	refresh();
 }
 
 void MemoryCardSettingsWidget::ejectSlot(u32 slot)
 {
-	m_dialog->setStringSettingValue("MemoryCards", getSlotFilenameKey(slot).c_str(), m_dialog->isPerGameSettings() ? nullptr : "");
+	dialog()->setStringSettingValue(CONFIG_SECTION, getSlotFilenameKey(slot).c_str(),
+		dialog()->isPerGameSettings() ? std::nullopt : std::optional<const char*>(""));
 	refresh();
 }
 
 void MemoryCardSettingsWidget::createCard()
 {
-	CreateMemoryCardDialog dialog(QtUtils::GetRootWidget(this));
+	MemoryCardCreateDialog dialog(QtUtils::GetRootWidget(this));
 	if (dialog.exec() == QDialog::Accepted)
 		refresh();
 }
@@ -188,29 +182,27 @@ QString MemoryCardSettingsWidget::getSelectedCard() const
 	return ret;
 }
 
+bool MemoryCardSettingsWidget::isSelectedCardFormatted() const
+{
+	const QList<QTreeWidgetItem*> selection(m_ui.cardList->selectedItems());
+	if (!selection.empty())
+		return selection[0]->data(0, Qt::UserRole).toBool();
+	return false;
+}
+
 void MemoryCardSettingsWidget::updateCardActions()
 {
 	QString selectedCard = getSelectedCard();
 	const bool hasSelection = !selectedCard.isEmpty();
-	
+
 	std::optional<AvailableMcdInfo> cardInfo = FileMcd_GetCardInfo(selectedCard.toStdString());
 	bool isPS1 = (cardInfo.has_value() ? cardInfo.value().file_type == MemoryCardFileType::PS1 : false);
 
 	m_ui.convertCard->setEnabled(hasSelection && !isPS1);
-	m_ui.duplicateCard->setEnabled(hasSelection);
 	m_ui.renameCard->setEnabled(hasSelection);
 	m_ui.deleteCard->setEnabled(hasSelection);
 }
 
-void MemoryCardSettingsWidget::duplicateCard()
-{
-	const QString selectedCard(getSelectedCard());
-	if (selectedCard.isEmpty())
-		return;
-
-	QMessageBox::critical(this, tr("Error"), tr("Not yet implemented."));
-}
-
 void MemoryCardSettingsWidget::deleteCard()
 {
 	const QString selectedCard(getSelectedCard());
@@ -218,7 +210,7 @@ void MemoryCardSettingsWidget::deleteCard()
 		return;
 
 	if (QMessageBox::question(QtUtils::GetRootWidget(this), tr("Delete Memory Card"),
-			tr("Are you sure you wish to delete the memory card '%1'?\n\n"
+			tr("Are you sure you wish to delete the Memory Card '%1'?\n\n"
 			   "This action cannot be reversed, and you will lose any saves on the card.")
 				.arg(selectedCard)) != QMessageBox::Yes)
 	{
@@ -228,7 +220,7 @@ void MemoryCardSettingsWidget::deleteCard()
 	if (!FileMcd_DeleteCard(selectedCard.toStdString()))
 	{
 		QMessageBox::critical(QtUtils::GetRootWidget(this), tr("Delete Memory Card"),
-			tr("Failed to delete the memory card. The log may have more information."));
+			tr("Failed to delete the Memory Card. The log may have more information."));
 		return;
 	}
 
@@ -241,15 +233,15 @@ void MemoryCardSettingsWidget::renameCard()
 	if (selectedCard.isEmpty())
 		return;
 
-	const QString newName(QInputDialog::getText(QtUtils::GetRootWidget(this),
-		tr("Rename Memory Card"), tr("New Card Name"), QLineEdit::Normal, selectedCard));
+	const QString newName(QInputDialog::getText(
+		QtUtils::GetRootWidget(this), tr("Rename Memory Card"), tr("New Card Name"), QLineEdit::Normal, selectedCard));
 	if (newName.isEmpty() || newName == selectedCard)
 		return;
 
 	if (!newName.endsWith(QStringLiteral(".ps2")) || newName.length() <= 4)
 	{
-		QMessageBox::critical(QtUtils::GetRootWidget(this), tr("Rename Memory Card"),
-			tr("New name is invalid, it must end with .ps2"));
+		QMessageBox::critical(
+			QtUtils::GetRootWidget(this), tr("Rename Memory Card"), tr("New name is invalid, it must end with .ps2"));
 		return;
 	}
 
@@ -264,7 +256,7 @@ void MemoryCardSettingsWidget::renameCard()
 	if (!FileMcd_RenameCard(selectedCard.toStdString(), newNameStr))
 	{
 		QMessageBox::critical(QtUtils::GetRootWidget(this), tr("Rename Memory Card"),
-			tr("Failed to rename memory card. The log may contain more information."));
+			tr("Failed to rename Memory Card. The log may contain more information."));
 		return;
 	}
 
@@ -274,12 +266,18 @@ void MemoryCardSettingsWidget::renameCard()
 void MemoryCardSettingsWidget::convertCard()
 {
 	const QString selectedCard(getSelectedCard());
-	
+
 	if (selectedCard.isEmpty())
 		return;
 
+	if (!isSelectedCardFormatted())
+	{
+		QMessageBox::critical(this, tr("Error"), tr("Cannot convert an unformatted memory card."));
+		return;
+	}
+
 	MemoryCardConvertDialog dialog(QtUtils::GetRootWidget(this), selectedCard);
-	
+
 	if (dialog.IsSetup() && dialog.exec() == QDialog::Accepted)
 		refresh();
 }
@@ -293,56 +291,58 @@ void MemoryCardSettingsWidget::listContextMenuRequested(const QPoint& pos)
 	{
 		for (u32 slot = 0; slot < MAX_SLOTS; slot++)
 		{
-			connect(menu.addAction(tr("Use for Port %1").arg(slot + 1)), &QAction::triggered,
-				this, [this, &selectedCard, slot]() { tryInsertCard(slot, selectedCard); });
+			connect(menu.addAction(tr("Use for Slot %1").arg(slot + 1)), &QAction::triggered, this,
+				[this, &selectedCard, slot]() { tryInsertCard(slot, selectedCard); });
 		}
 		menu.addSeparator();
 
-		connect(menu.addAction(tr("Duplicate")), &QAction::triggered, this, &MemoryCardSettingsWidget::duplicateCard);
 		connect(menu.addAction(tr("Rename")), &QAction::triggered, this, &MemoryCardSettingsWidget::renameCard);
 		connect(menu.addAction(tr("Convert")), &QAction::triggered, this, &MemoryCardSettingsWidget::convertCard);
 		connect(menu.addAction(tr("Delete")), &QAction::triggered, this, &MemoryCardSettingsWidget::deleteCard);
 		menu.addSeparator();
 	}
 
-	connect(menu.addAction("Create"), &QAction::triggered, this, &MemoryCardSettingsWidget::createCard);
+	connect(menu.addAction(tr("Create")), &QAction::triggered, this, &MemoryCardSettingsWidget::createCard);
 
 	menu.exec(m_ui.cardList->mapToGlobal(pos));
 }
 
 void MemoryCardSettingsWidget::refresh()
 {
+	const bool perGame = dialog()->isPerGameSettings();
+
 	for (u32 slot = 0; slot < static_cast<u32>(m_slots.size()); slot++)
 	{
 		const bool enabled = m_slots[slot].enable->isChecked();
+		const std::string slotKey = getSlotFilenameKey(slot);
 		const std::optional<std::string> name(
-			m_dialog->getStringValue("MemoryCards", getSlotFilenameKey(slot).c_str(),
-				FileMcd_GetDefaultName(slot).c_str()));
+			dialog()->getEffectiveStringValue(CONFIG_SECTION, slotKey.c_str(), FileMcd_GetDefaultName(slot).c_str()));
+		const bool inherited = perGame ? !dialog()->containsSettingValue(CONFIG_SECTION, slotKey.c_str()) : false;
 
-		m_slots[slot].slot->setCard(name);
+		m_slots[slot].slot->setCard(name, inherited);
 		m_slots[slot].slot->setEnabled(enabled);
 		m_slots[slot].eject->setEnabled(enabled);
 	}
 
-	m_ui.cardList->refresh(m_dialog);
+	m_ui.cardList->refresh(dialog());
 	updateCardActions();
 }
 
 void MemoryCardSettingsWidget::swapCards()
 {
-	const std::string card_1_key(getSlotFilenameKey(0));
-	const std::string card_2_key(getSlotFilenameKey(1));
-	std::optional<std::string> card_1_name(m_dialog->getStringValue("MemoryCards", card_1_key.c_str(), std::nullopt));
-	std::optional<std::string> card_2_name(m_dialog->getStringValue("MemoryCards", card_2_key.c_str(), std::nullopt));
-	if (!card_1_name.has_value() || card_1_name->empty() ||
-		!card_2_name.has_value() || card_2_name->empty())
+	const std::string card1Key = getSlotFilenameKey(0);
+	const std::string card2Key = getSlotFilenameKey(1);
+	std::optional<std::string> card1Name = dialog()->getStringValue(CONFIG_SECTION, card1Key.c_str(), std::nullopt);
+	std::optional<std::string> card2Name = dialog()->getStringValue(CONFIG_SECTION, card2Key.c_str(), std::nullopt);
+	if (!card1Name.has_value() || card1Name->empty() || !card2Name.has_value() || card2Name->empty())
 	{
-		QMessageBox::critical(QtUtils::GetRootWidget(this), tr("Error"), tr("Both ports must have a card selected to swap."));
+		QMessageBox::critical(
+			QtUtils::GetRootWidget(this), tr("Error"), tr("Both slots must have a card selected to swap."));
 		return;
 	}
 
-	m_dialog->setStringSettingValue("MemoryCards", card_1_key.c_str(), card_2_name->c_str());
-	m_dialog->setStringSettingValue("MemoryCards", card_2_key.c_str(), card_1_name->c_str());
+	dialog()->setStringSettingValue(CONFIG_SECTION, card1Key.c_str(), card2Name->c_str());
+	dialog()->setStringSettingValue(CONFIG_SECTION, card2Key.c_str(), card1Name->c_str());
 	refresh();
 }
 
@@ -385,7 +385,7 @@ static QString getSizeSummary(const AvailableMcdInfo& mcd)
 static QIcon getCardIcon(const AvailableMcdInfo& mcd)
 {
 	if (mcd.type == MemoryCardType::File)
-		return QIcon::fromTheme(QStringLiteral("sd-card-line"));
+		return QIcon::fromTheme(QStringLiteral("memcard-line"));
 	else
 		return QIcon::fromTheme(QStringLiteral("folder-open-line"));
 }
@@ -425,7 +425,7 @@ void MemoryCardListWidget::mouseMoveEvent(QMouseEvent* event)
 	drag->exec(Qt::CopyAction);
 }
 
-void MemoryCardListWidget::refresh(SettingsDialog* dialog)
+void MemoryCardListWidget::refresh(SettingsWindow* dialog)
 {
 	clear();
 
@@ -437,8 +437,8 @@ void MemoryCardListWidget::refresh(SettingsDialog* dialog)
 	std::array<std::string, MemoryCardSettingsWidget::MAX_SLOTS> currentCards;
 	for (u32 i = 0; i < static_cast<u32>(currentCards.size()); i++)
 	{
-		const std::optional<std::string> filename = dialog->getStringValue("MemoryCards",
-			getSlotFilenameKey(i).c_str(), FileMcd_GetDefaultName(i).c_str());
+		const std::optional<std::string> filename = dialog->getEffectiveStringValue(
+			CONFIG_SECTION, getSlotFilenameKey(i).c_str(), FileMcd_GetDefaultName(i).c_str());
 		if (filename.has_value())
 			currentCards[i] = std::move(filename.value());
 	}
@@ -455,6 +455,10 @@ void MemoryCardListWidget::refresh(SettingsDialog* dialog)
 		item->setText(1, getSizeSummary(mcd));
 		item->setText(2, mcd.formatted ? tr("Yes") : tr("No"));
 		item->setText(3, mtime.toString(QLocale::system().dateTimeFormat(QLocale::ShortFormat)));
+
+		// store formatted metadata
+		item->setData(0, Qt::UserRole, mcd.formatted);
+
 		addTopLevelItem(item);
 	}
 }
@@ -464,6 +468,8 @@ MemoryCardSlotWidget::MemoryCardSlotWidget(QWidget* parent)
 {
 	setAcceptDrops(true);
 	setSelectionMode(NoSelection);
+	setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+	setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
 }
 
 MemoryCardSlotWidget::~MemoryCardSlotWidget() = default;
@@ -492,7 +498,7 @@ void MemoryCardSlotWidget::dropEvent(QDropEvent* event)
 	emit cardDropped(text);
 }
 
-void MemoryCardSlotWidget::setCard(const std::optional<std::string>& name)
+void MemoryCardSlotWidget::setCard(const std::optional<std::string>& name, bool inherited)
 {
 	clear();
 	if (!name.has_value() || name->empty())
@@ -509,6 +515,17 @@ void MemoryCardSlotWidget::setCard(const std::optional<std::string>& name)
 	else
 	{
 		item->setIcon(QIcon::fromTheme("close-line"));
+		//: Ignore Crowdin's warning for [Missing], the text should be translated.
 		item->setText(tr("%1 [Missing]").arg(QString::fromStdString(name.value())));
 	}
+
+	if (inherited)
+	{
+		QFont font = item->font();
+		font.setItalic(true);
+		item->setFont(font);
+		item->setForeground(palette().brush(QPalette::Disabled, QPalette::Text));
+	}
+
+	item->setToolTip(item->text());
 }
