@@ -28,6 +28,9 @@ public:
 	IsoHasher();
 	~IsoHasher();
 
+	IsoHasher(const IsoHasher&) = delete;
+	IsoHasher& operator=(const IsoHasher&) = delete;
+
 	static std::string_view GetTrackTypeString(u32 type);
 
 	u32 GetTrackCount() const { return static_cast<u32>(m_tracks.size()); }
@@ -44,6 +47,7 @@ private:
 	bool ComputeTrackHash(Track& track, ProgressCallback* callback);
 
 	std::vector<Track> m_tracks;
+	bool m_is_locked = false;
 	bool m_is_open = false;
 	bool m_is_cd = false;
 };
