@@ -163,6 +163,9 @@ void SymbolImporter::Reset()
 
 void SymbolImporter::LoadAndAnalyseElf(Pcsx2Config::DebugAnalysisOptions options)
 {
+	if (!VMManager::HasValidVM())
+		return;
+
 	const std::string& elf_path = VMManager::GetCurrentELF();
 
 	Error error;
@@ -192,6 +195,9 @@ void SymbolImporter::AnalyseElf(
 	Pcsx2Config::DebugAnalysisOptions options,
 	bool wait_until_elf_is_loaded)
 {
+	if (!VMManager::HasValidVM())
+		return;
+
 	// Search for a .sym file to load symbols from.
 	std::string nocash_path;
 	CDVD_SourceType source_type = CDVDsys_GetSourceType();
@@ -311,9 +317,10 @@ void SymbolImporter::ClearExistingSymbols(ccc::SymbolDatabase& database, const P
 	{
 		bool should_destroy = ShouldClearSymbolsFromSourceByDefault(source.name());
 
-		for (const DebugSymbolSource& source_config : options.SymbolSources)
-			if (source_config.Name == source.name())
-				should_destroy = source_config.ClearDuringAnalysis;
+		if (!options.AutomaticallySelectSymbolsToClear)
+			for (const DebugSymbolSource& source_config : options.SymbolSources)
+				if (source_config.Name == source.name())
+					should_destroy = source_config.ClearDuringAnalysis;
 
 		if (should_destroy)
 			sources_to_destroy.emplace_back(source.handle());
@@ -326,9 +333,9 @@ void SymbolImporter::ClearExistingSymbols(ccc::SymbolDatabase& database, const P
 bool SymbolImporter::ShouldClearSymbolsFromSourceByDefault(const std::string& source_name)
 {
 	return source_name.find("Symbol Table") != std::string::npos ||
-		   source_name == "ELF Section Headers" ||
-		   source_name == "Function Scanner" ||
-		   source_name == "Nocash Symbols";
+	       source_name == "ELF Section Headers" ||
+	       source_name == "Function Scanner" ||
+	       source_name == "Nocash Symbols";
 }
 
 void SymbolImporter::ImportSymbols(
