@@ -88,12 +88,28 @@ protected:
 
 	__forceinline bool HasEdge() const { return (m_draw_edge != nullptr); }
 
+	template <bool step_x, bool pos_x, bool pos_y, bool tl, bool side>
+	void DrawEdgeTriangle(const GSVertexSW& v0, const GSVertexSW& v1, const GSVertexSW& dv,
+		const GSVector4i& efun1, const GSVector4i& efun2);
+	template <bool step_x, bool pos_x, bool pos_y, bool aa>
+	void DrawEdgeLine(const GSVertexSW& v0, const GSVertexSW& v1, const GSVertexSW& dv);
+	void DrawEdgeTriangle(const GSVertexSW& v0, const GSVertexSW& v1, const GSVertexSW& dv,
+		const GSVector4i& efun1, const GSVector4i& efun2, bool topleft);
+	void DrawEdgeLine(const GSVertexSW& v0, const GSVertexSW& v1, const GSVertexSW& dv, bool has_edge);
+	
 	template <bool scissor_test>
 	void DrawPoint(const GSVertexSW* vertex, int vertex_count, const u16* index, int index_count);
 	void DrawLine(const GSVertexSW* vertex, const u16* index);
 	void DrawTriangle(const GSVertexSW* vertex, const u16* index);
 	void DrawSprite(const GSVertexSW* vertex, const u16* index);
 
+	using DrawEdgeTrianglePtr = void (GSRasterizer::*)(const GSVertexSW& , const GSVertexSW&, const GSVertexSW&,
+		const GSVector4i&, const GSVector4i&);
+	static const DrawEdgeTrianglePtr m_draw_edge_triangle[2][2][2][2][2];
+
+	using DrawEdgeLinePtr = void (GSRasterizer::*)(const GSVertexSW&, const GSVertexSW&, const GSVertexSW&);
+	static const DrawEdgeLinePtr m_draw_edge_line[2][2][2][2];
+
 #if _M_SSE >= 0x501
 	__forceinline void DrawTriangleSection(int top, int bottom, GSVertexSW2& RESTRICT edge, const GSVertexSW2& RESTRICT dedge, const GSVertexSW2& RESTRICT dscan, const GSVector4& RESTRICT p0);
 #else
