@@ -1,23 +1,12 @@
-/*  PCSX2 - PS2 Emulator for PCs
- *  Copyright (C) 2002-2022  PCSX2 Dev Team
- *
- *  PCSX2 is free software: you can redistribute it and/or modify it under the terms
- *  of the GNU Lesser General Public License as published by the Free Software Found-
- *  ation, either version 3 of the License, or (at your option) any later version.
- *
- *  PCSX2 is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
- *  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR
- *  PURPOSE.  See the GNU General Public License for more details.
- *
- *  You should have received a copy of the GNU General Public License along with PCSX2.
- *  If not, see <http://www.gnu.org/licenses/>.
- */
+// SPDX-FileCopyrightText: 2002-2025 PCSX2 Dev Team
+// SPDX-License-Identifier: GPL-3.0+
 
 #pragma once
 
 #include "common/ProgressCallback.h"
 
 #ifdef _WIN32
+#include "common/RedtapeWindows.h"
 #include "7z.h"
 #include "7zFile.h"
 #endif
@@ -45,7 +34,7 @@ public:
 	std::string FindPCSX2Exe() const;
 
 private:
-	static bool RecursiveDeleteDirectory(const char* path);
+	bool RecursiveDeleteDirectory(const char* path);
 
 	void CloseUpdateZip();
 
